﻿using System.Data;
using MySqlConnector;

namespace MesETL.Shared.Helper;

public static class DatabaseHelper
{
    public static MySqlConnection CreateConnection(string connStr)
    {
        var newConnStr = new MySqlConnectionStringBuilder(connStr)
        {
            ConnectionTimeout = 30,
            DefaultCommandTimeout = 0,
        }.ConnectionString;
        return new MySqlConnection(newConnStr);
    }
    
    public static async Task<DataSet> QueryTableAsync(string connStr, string sql, CancellationToken ct = default)
    {
        await using var conn = CreateConnection(connStr);
        if(conn.State is not ConnectionState.Open)
            await conn.OpenAsync(ct);
        await using var cmd = conn.CreateCommand();
        cmd.CommandText = sql;
        var ds = new DataSet();
        new MySqlDataAdapter(cmd).Fill(ds);
        return ds;
    }

    public static async Task<object?> QueryScalarAsync(string connStr, string sql, CancellationToken ct = default)
    {
        await using var conn = CreateConnection(connStr);
        if(conn.State is not ConnectionState.Open)
            await conn.OpenAsync(ct);
        await using var cmd = conn.CreateCommand();
        cmd.CommandText = sql;
        return await cmd.ExecuteScalarAsync(ct);
    }

    public static async Task<int> NonQueryAsync(string connStr, string sql, CancellationToken ct = default)
    {
        await using var conn = CreateConnection(connStr);
        if(conn.State is not ConnectionState.Open)
            await conn.OpenAsync(ct);
        await using var cmd = conn.CreateCommand();
        cmd.CommandText = sql;
        return await cmd.ExecuteNonQueryAsync(ct);
    }

    public static async Task<int> TransactionAsync(string connStr, string sql, params MySqlParameter[] parameters)
    {
        await using var conn = CreateConnection(connStr);
        if(conn.State is not ConnectionState.Open)
            await conn.OpenAsync();
        await using var trans = await conn.BeginTransactionAsync();
        await using var cmd = conn.CreateCommand();
        cmd.CommandText = sql;
        cmd.Transaction = trans;
        cmd.Parameters.AddRange(parameters);
        var rows = await cmd.ExecuteNonQueryAsync();
        await trans.CommitAsync();
        return rows;
    }
}